@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -46,7 +46,7 @@ from beartype._util.py.utilpyversion import (
     IS_PYTHON_AT_LEAST_3_9,
     IS_PYTHON_AT_LEAST_3_7,
 )
-from beartype._util.utiltyping import TupleTypes
+from beartype._data.datatyping import TupleTypes
 from typing import Any, Optional
 
 # See the "beartype.cave" submodule for further commentary.
@@ -505,6 +505,14 @@ def get_hint_pep_sign_or_none(hint: Any) -> Optional[HintSign]:
     #     >>> bad_guy_type_hint.__module__ = 'typing'
     #     >>> repr(bad_guy_type_hint)
     #     typing.List   # <---- this is genuine bollocks
+    #
+    # Likewise, some of these classes define __repr__() methods prefixed by the
+    # machine-readable representations of their children. Again, to avoid false
+    # positives, this phase *MUST* thus be performed before repr()-based tests
+    # regardless of efficiency concerns: e.g.,
+    #     # Under Python >= 3.10:
+    #     >>> repr(tuple[str, ...] | bool)
+    #     tuple[str, ...] | bool  # <---- this is fine but *NOT* a tuple!
 
     # Class of this hint.
     hint_type = hint.__class__
@@ -716,11 +724,11 @@ get_hint_pep_origin_or_none.__doc__ = '''
 
     Caveats
     ----------
-    **The high-level** :func:`get_hint_pep_type_isinstanceable_or_none` function should
-    always be called in lieu of this low-level function.** Whereas the former
-    is guaranteed to return either a class or ``None``, this function enjoys no
-    such guarantees and instead returns what the caller can only safely assume
-    to be an arbitrary object.
+    **The high-level** :func:`get_hint_pep_type_isinstanceable_or_none`
+    function should always be called in lieu of this low-level function.**
+    Whereas the former is guaranteed to return either a class or ``None``, this
+    function enjoys no such guarantees and instead returns what the caller can
+    only safely assume to be an arbitrary object.
 
     If this function *must* be called, **this function should always be called
     in lieu of attempting to directly access the low-level** ``__origin__``
