@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -55,6 +55,7 @@ from beartype._data.hint.pep.sign.datapepsigns import (
     HintSignAnnotated,
     HintSignNewType,
     HintSignNumpyArray,
+    HintSignDataclassInitVar,
     HintSignType,
     HintSignTypeVar,
     HintSignTypedDict,
@@ -64,9 +65,11 @@ from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.pep.proposal.pep484.utilpep484union import (
     make_hint_pep484_union)
 from beartype._util.hint.pep.proposal.utilpep544 import (
-    reduce_hint_pep484_generic_io_to_pep544_protocol,
     is_hint_pep484_generic_io,
+    reduce_hint_pep484_generic_io_to_pep544_protocol,
 )
+from beartype._util.hint.pep.proposal.utilpep557 import (
+    get_hint_pep557_initvar_arg)
 from beartype._util.hint.pep.proposal.utilpep585 import is_hint_pep585_builtin
 from beartype._util.hint.pep.utilpepget import get_hint_pep_sign_or_none
 from beartype._util.hint.utilhinttest import die_unless_hint
@@ -117,11 +120,6 @@ rather than the :func:`functools.lru_cache` decorator. Why? Because:
 '''
 
 # ....................{ SANIFIERS                         }....................
-#FIXME: Define a new sanify_hint_child() function internally
-#calling both coerce_hint_child() and _reduce_hint().
-#FIXME: Replace *ALL* external calls to _reduce_hint() with calls to
-#coerce_hint_child() instead.
-
 #FIXME: Unit test us up, please.
 #FIXME: Revise docstring in accordance with recent dramatic improvements.
 def sanify_hint_root(
@@ -198,8 +196,6 @@ def sanify_hint_root(
         * A PEP-noncompliant type hint.
         * A supported PEP-compliant type hint.
     '''
-    assert callable(func), f'{repr(func)} not callable.'
-    assert isinstance(pith_name, str), f'{pith_name} not string.'
 
     # PEP-compliant type hint coerced (i.e., permanently converted in the
     # annotations dunder dictionary of the passed callable) from this possibly
@@ -273,8 +269,7 @@ def sanify_hint_child(hint: object, exception_prefix: str) -> Any:
     # Return this hint first coerced and then reduced, intentionally covering
     # the subset of the logic performed by the sanify_hint_root() sanifier
     # specifically applicable to child type hints.
-    return _reduce_hint(
-        _coerce_hint_any(hint, exception_prefix), exception_prefix)
+    return _reduce_hint(_coerce_hint_any(hint), exception_prefix)
 
 # ....................{ COERCERS                          }....................
 #FIXME: Document mypy-specific coercion in the docstring as well, please.
@@ -414,10 +409,10 @@ def _coerce_hint_root(
     # position relative to other type hints.
     #
     # Return this hint, possibly coerced as a context-agnostic type hint.
-    return _coerce_hint_any(hint=hint, exception_prefix=exception_prefix)
+    return _coerce_hint_any(hint)
 
 
-def _coerce_hint_any(hint: object, exception_prefix: str) -> Any:
+def _coerce_hint_any(hint: object) -> Any:
     '''
     PEP-compliant type hint coerced (i.e., converted) from the passed
     PEP-compliant type hint if this hint is coercible *or* this hint as is
@@ -530,9 +525,6 @@ def _coerce_hint_any(hint: object, exception_prefix: str) -> Any:
     ----------
     hint : object
         Type hint to be possibly coerced.
-    exception_prefix : str
-        Human-readable label prefixing the representation of this object in the
-        exception message.
 
     Returns
     ----------
@@ -557,22 +549,18 @@ def _coerce_hint_any(hint: object, exception_prefix: str) -> Any:
     # * Else, one or more prior copies of this hint have already been passed to
     #   this function. In this case, replace this subsequent copy by the first
     #   copy of this hint originally passed to a prior call of this function.
-
-    #FIXME: *UNIT TEST US UP, PLEASE.* This requires significant testing.
     if is_hint_pep585_builtin(hint):
-        return _HINT_REPR_TO_HINT.get_value_static(
-            key=repr(hint), value=hint)
+        return _HINT_REPR_TO_HINT.get_value_static(key=repr(hint), value=hint)
     # Else, this hint is *NOT* PEP 585-compliant.
 
     # Return this uncoerced hint as is.
     return hint
 
 # ....................{ REDUCERS                          }....................
+#FIXME: Improve documentation to list all reductions performed by this reducer.
+#Sadly, this documentation is currently quite out-of-date. What? It happens!
 @callable_cached
-def _reduce_hint(
-    hint: Any,
-    exception_prefix: str,
-) -> object:
+def _reduce_hint(hint: Any, exception_prefix: str) -> object:
     '''
     Lower-level type hint reduced (i.e., converted) from the passed
     higher-level type hint if this hint is reducible *or* this hint as is
@@ -583,7 +571,7 @@ def _reduce_hint(
     * *Not* PEP-compliant, this hint is returned as is unmodified.
     * PEP 593-compliant (i.e., :class:`typing.Annotated`) but beartype-agnostic
       (i.e., its second argument is *not* an instance of the
-      :class:`beartype.vale._valevale.BeartypeValidator` class produced by
+      :class:`beartype.vale._core._valecore.BeartypeValidator` class produced by
       subscripting the :class:`beartype.vale.Is` class), this hint is reduced
       to the first argument subscripting this hint. Doing so ignores *all*
       irrelevant annotations on this hint (e.g., reducing
@@ -765,7 +753,18 @@ def _reduce_hint(
     # from third-party classes). Ergo, we can neither safely emit warnings nor
     # raise exceptions on visiting these classes under *ANY* Python version.
     elif is_hint_pep484_generic_io(hint):
-        hint = reduce_hint_pep484_generic_io_to_pep544_protocol(hint)
+        hint = reduce_hint_pep484_generic_io_to_pep544_protocol(
+            hint=hint, exception_prefix=exception_prefix)
+    # ..................{ PEP 557                           }..................
+    # If this hint is a dataclass-specific initialization-only instance
+    # variable (i.e., instance of the PEP 557-compliant "dataclasses.InitVar"
+    # class introduced by Python 3.8.0), reduce this functionally useless hint
+    # to the functionally useful child type hint subscripting this parent hint.
+    #
+    # "InitVar" instances are stupefyingly rare and thus detected even later.
+    elif hint_sign is HintSignDataclassInitVar:
+        hint = get_hint_pep557_initvar_arg(
+            hint=hint, exception_prefix=exception_prefix)
 
     # Return this possibly reduced hint.
     return hint
