@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2021 Beartype authors.
+# Copyright (c) 2014-2022 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -35,21 +35,24 @@ def add_data(data_module: 'ModuleType') -> None:
 
     # ..................{ IMPORTS                           }..................
     # Defer attribute-dependent imports.
+    from beartype.typing import List
     from beartype.vale import (
         Is,
         IsAttr,
         IsEqual,
+        IsInstance,
         IsSubclass,
     )
     from beartype._data.hint.pep.sign.datapepsigns import (
         HintSignAnnotated,
         HintSignList,
     )
-    from beartype._util.hint.pep.utilpepattr import (
-        HINT_ATTR_LIST,
-    )
     from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_7
-    from beartype_test.a00_unit.data.data_type import Class, Subclass
+    from beartype_test.a00_unit.data.data_type import (
+        Class,
+        Subclass,
+        SubclassSubclass,
+    )
     from beartype_test.a00_unit.data.hint.util.data_hintmetacls import (
         HintPepMetadata,
         HintPithSatisfiedMetadata,
@@ -79,8 +82,7 @@ def add_data(data_module: 'ModuleType') -> None:
 
         # Deeply ignorable PEP 484-, 585-, and 593-compliant type hint
         # exercising numerous edge cases broken under prior releases.
-        Union[str, HINT_ATTR_LIST[int], NewType(
-            'MetaType', Annotated[object, 53])],
+        Union[str, List[int], NewType('MetaType', Annotated[object, 53])],
     ))
 
     # ..................{ TUPLES                            }..................
@@ -92,12 +94,10 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=Annotated[str, int],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant.
                 HintPithSatisfiedMetadata(
                     'Towards a timely, wines‐enticing gate'),
-            ),
-            piths_unsatisfied_meta=(
                 # List of string constants.
                 HintPithUnsatisfiedMetadata([
                     'Of languished anger’s sap‐spated rushings',]),
@@ -106,16 +106,14 @@ def add_data(data_module: 'ModuleType') -> None:
 
         # Annotated of a "typing" type.
         HintPepMetadata(
-            hint=Annotated[HINT_ATTR_LIST[str], int],
+            hint=Annotated[List[str], int],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # List of string constants.
                 HintPithSatisfiedMetadata([
                     'MINERVA‐unnerving, verve‐sapping enervations',
                     'Of a magik-stoned Shinto rivery',
                 ]),
-            ),
-            piths_unsatisfied_meta=(
                 # String constant.
                 HintPithUnsatisfiedMetadata('Of a Spicily sated',),
             ),
@@ -180,12 +178,10 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=AnnotatedStrIsLength,
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant satisfying this validator.
                 HintPithSatisfiedMetadata(
                     'To Ɯṙaith‐like‐upwreathed ligaments'),
-            ),
-            piths_unsatisfied_meta=(
                 # Byte-string constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith=b'Down-bound',
@@ -205,12 +201,10 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=Annotated[str, IsLengthy, IsSentence, IsQuoted],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant satisfying these validators.
                 HintPithSatisfiedMetadata(
                     '"Into piezo‐electrical, dun‐dappled lights" and...'),
-            ),
-            piths_unsatisfied_meta=(
                 # Byte-string constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith=b'Joy.',
@@ -240,12 +234,10 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=Annotated[str, IsLengthy & IsSentence & IsQuoted],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant satisfying these validators.
                 HintPithSatisfiedMetadata(
                     '"Into piezo‐electrical, dun‐dappled lights" and...'),
-            ),
-            piths_unsatisfied_meta=(
                 # Byte-string constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith=b'Joy.',
@@ -274,12 +266,10 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=Annotated[str, IsLengthyOrUnquotedSentence],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant satisfying these validators.
                 HintPithSatisfiedMetadata(
                     'Dialectical, eclectic mind‐toys'),
-            ),
-            piths_unsatisfied_meta=(
                 # Byte-string constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith=b'Of Cycladic impoverishment, cyclically unreeling',
@@ -309,16 +299,14 @@ def add_data(data_module: 'ModuleType') -> None:
         HintPepMetadata(
             hint=Annotated[Annotated[str, IsLengthy], IsSentence],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # String constant satisfying these validators.
                 HintPithSatisfiedMetadata(
                     'Antisatellite‐dendroidal, Θṙbital Cemetery orbs — '
                     'of Moab.'
                 ),
-            ),
-            piths_unsatisfied_meta=(
                 # Byte-string constant *NOT* an instance of the expected type.
-                HintPithUnsatisfiedMetadata(pith=b'Then, and'),
+                HintPithUnsatisfiedMetadata(b'Then, and'),
                 # String constant violating only the first of these validators.
                 HintPithUnsatisfiedMetadata('Though a...'),
             ),
@@ -327,18 +315,16 @@ def add_data(data_module: 'ModuleType') -> None:
         # List of annotateds of isinstanceable types annotated by one
         # beartype-specific validator defined as a lambda function.
         HintPepMetadata(
-            hint=HINT_ATTR_LIST[AnnotatedStrIsLength],
+            hint=List[AnnotatedStrIsLength],
             pep_sign=HintSignList,
             isinstanceable_type=list,
-            is_pep585_builtin=HINT_ATTR_LIST is list,
-            piths_satisfied_meta=(
+            is_pep585_builtin=List is list,
+            piths_meta=(
                 # List of string constants satisfying this validator.
                 HintPithSatisfiedMetadata([
                     'An‐atomically Island‐stranded, adrift land)',
                     'That You randily are That worm‐tossed crabapple of',
                 ]),
-            ),
-            piths_unsatisfied_meta=(
                 # String constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith="Our Sturm‐sapped disorder's convolution of",
@@ -363,11 +349,9 @@ def add_data(data_module: 'ModuleType') -> None:
             hint=Annotated[
                 CathecticallyEnsconceYouIn, IsAttrThisMobbedTristeOf],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # Instance of this class satisfying this validator.
                 HintPithSatisfiedMetadata(BOSS_EMBOSSED_ORDERING),
-            ),
-            piths_unsatisfied_meta=(
                 # String constant *NOT* an instance of this class.
                 HintPithUnsatisfiedMetadata(
                     pith='An atoll nuclear broilers newly cleared of',
@@ -385,16 +369,14 @@ def add_data(data_module: 'ModuleType') -> None:
         # Annotated of an isinstanceable type annotated by one
         # beartype-specific equality validator.
         HintPepMetadata(
-            hint=Annotated[HINT_ATTR_LIST[str], IsEqual[AMPLY_IMPISH]],
+            hint=Annotated[List[str], IsEqual[AMPLY_IMPISH]],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # Exact object subscripting this validator.
                 HintPithSatisfiedMetadata(AMPLY_IMPISH),
                 # Object *NOT* subscripting this validator but equal to this
                 # object.
                 HintPithSatisfiedMetadata(AMPLY_IMPISH[:]),
-            ),
-            piths_unsatisfied_meta=(
                 # String constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith='May Your coarsest, Incessantly cast‐off jobs of a',
@@ -411,19 +393,50 @@ def add_data(data_module: 'ModuleType') -> None:
             ),
         ),
 
+        # ................{ ANNOTATED ~ beartype : isinstance }................
+        # Annotated of an isinstanceable type annotated by one
+        # beartype-specific type instance validator.
+        HintPepMetadata(
+            hint=Annotated[Class, ~IsInstance[SubclassSubclass]],
+            pep_sign=HintSignAnnotated,
+            piths_meta=(
+                # Instance of the class subscripting this validator.
+                HintPithSatisfiedMetadata(Class()),
+                # Instance of the subclass subclassing the class subscripting
+                # this validator.
+                HintPithSatisfiedMetadata(Subclass()),
+                # Instance of the subsubclass subclassing the subclass
+                # subclassing the class subscripting this validator.
+                HintPithUnsatisfiedMetadata(SubclassSubclass()),
+                # Class subscripting this validator.
+                HintPithUnsatisfiedMetadata(Class),
+                # String constant *NOT* an instance of the expected type.
+                HintPithUnsatisfiedMetadata(
+                    pith=(
+                        'Architrave‐contravening, '
+                        'indigenously chitinous tactilities) of'
+                    ),
+                    # Match that the exception message raised for this object
+                    # embeds a classname in the expected list.
+                    exception_str_match_regexes=(r'\bClass\b',),
+                ),
+            ),
+        ),
+
         # ................{ ANNOTATED ~ beartype : issubclass }................
         # Annotated of an isinstanceable type annotated by one
-        # beartype-specific equality validator.
+        # beartype-specific type inheritance validator.
         HintPepMetadata(
             hint=Annotated[type, IsSubclass[Class]],
             pep_sign=HintSignAnnotated,
-            piths_satisfied_meta=(
+            piths_meta=(
                 # Class subscripting this validator.
                 HintPithSatisfiedMetadata(Class),
                 # Class subclassing the class subscripting this validator.
                 HintPithSatisfiedMetadata(Subclass),
-            ),
-            piths_unsatisfied_meta=(
+                # Class *NOT* subclassing the class subscripting this
+                # validator.
+                HintPithUnsatisfiedMetadata(str),
                 # String constant *NOT* an instance of the expected type.
                 HintPithUnsatisfiedMetadata(
                     pith='May Your coarsest, Incessantly cast‐off jobs of a',
@@ -431,9 +444,6 @@ def add_data(data_module: 'ModuleType') -> None:
                     # embeds a classname in the expected list.
                     exception_str_match_regexes=(r'\bClass\b',),
                 ),
-                # Class *NOT* subclassing the class subscripting this
-                # validator.
-                HintPithUnsatisfiedMetadata(str),
             ),
         ),
     ))
@@ -451,12 +461,10 @@ def add_data(data_module: 'ModuleType') -> None:
             HintPepMetadata(
                 hint=Annotated[str, []],
                 pep_sign=HintSignAnnotated,
-                piths_satisfied_meta=(
+                piths_meta=(
                     # String constant.
                     HintPithSatisfiedMetadata(
                         'Papally Ľust‐besmirched Merchet laws'),
-                ),
-                piths_unsatisfied_meta=(
                     # List of string constants.
                     HintPithUnsatisfiedMetadata([
                         "Of our ôver‐crowdedly cowed crowd's opinion‐",]),
